@@ -25,12 +25,9 @@ import rpm
 import copy
 import urlparse
 from parser import ConfigPreProcessor
-try:
-    from iniparse.compat import NoSectionError, NoOptionError, ConfigParser
-    from iniparse.compat import ParsingError
-except ImportError:
-    from ConfigParser import NoSectionError, NoOptionError, ConfigParser
-    from ConfigParser import ParsingError
+from iniparse import INIConfig
+from iniparse.compat import NoSectionError, NoOptionError, ConfigParser
+from iniparse.compat import ParsingError
 import rpmUtils.transaction
 import rpmUtils.arch
 import Errors
@@ -322,6 +319,9 @@ class BoolOption(Option):
             return "0"
 
 class FloatOption(Option):
+    """
+    An option representing a numeric float value.
+    """
     def parse(self, s):
         try:
             return float(s.strip())
@@ -395,6 +395,11 @@ class BytesOption(Option):
 
 class ThrottleOption(BytesOption):
 
+    """
+    An option representing a bandwidth throttle value. See
+    ThrottleOption.parse for acceptable input values.
+    """
+
     def parse(self, s):
         """Get a throttle option. 
 
@@ -562,6 +567,7 @@ class StartupConf(BaseConfig):
     plugins = BoolOption(False)
     pluginpath = ListOption(['/usr/share/yum-plugins', '/usr/lib/yum-plugins'])
     pluginconfpath = ListOption(['/etc/yum/pluginconf.d'])
+    gaftonmode = BoolOption(False)
 
 class YumConf(StartupConf):
     '''
@@ -742,7 +748,10 @@ def readMainConfig(startupconf):
     
     # items related to the originating config file
     yumconf.config_file_path = startupconf.config_file_path
-    yumconf.config_file_age = os.stat(startupconf.config_file_path)[8]
+    if os.path.exists(startupconf.config_file_path):
+        yumconf.config_file_age = os.stat(startupconf.config_file_path)[8]
+    else:
+        yumconf.config_file_age = 0
     
     # propagate the debuglevel and errorlevel values:
     yumconf.debuglevel = startupconf.debuglevel
@@ -803,11 +812,28 @@ def _getsysver(installroot, distroverpkg):
     del ts
     return releasever
 
+def writeRawRepoFile(repo,only=None):
+    """
+    Writes changes in a repo object back to a .repo file.
+    @param repo: Repo Object
+    @param only: List of attributes to work on (None = All)
+    It work by reading the repo file, changes the values there shall be changed and write it back to disk.
+    """
+    ini = INIConfig(open(repo.repofile))
+    # Updated the ConfigParser with the changed values    
+    cfgOptions = repo.cfg.options(repo.id)
+    for name,value in repo.iteritems():
+        option = repo.optionobj(name)
+        if option.default != value or name in cfgOptions :
+            if only == None or name in only:
+                ini[repo.id][name] = option.tostring(value)
+    fp =file(repo.repofile,"w")               
+    fp.write(str(ini))
+    fp.close()
+
 #def main():
-#    mainconf = readMainConfig('/etc/yum/yum.conf', '/')
-#    repoconf = readRepoConfig(mainconf.cfg, 'core', mainconf)
-#
-#    print `repoconf.name`
+#    mainconf = readMainConfig(readStartupConfig('/etc/yum/yum.conf', '/'))
+#    print mainconf.cachedir
 #
 #if __name__ == '__main__':
 #    main()
