@@ -12,8 +12,13 @@
 # along with this program; if not, write to the Free Software
 # Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
+"""
+Yum constants. Usually dealing with rpm magic numbers.
+"""
 
 #Constants
+YUM_PID_FILE = '/var/run/yum.pid'
+
 #transaction set states
 TS_UPDATE = 10
 TS_INSTALL = 20
@@ -22,10 +27,10 @@ TS_ERASE = 40
 TS_OBSOLETED = 50
 TS_OBSOLETING = 60
 TS_AVAILABLE = 70
-
+TS_UPDATED = 90
 
 TS_INSTALL_STATES = [TS_INSTALL, TS_TRUEINSTALL, TS_UPDATE, TS_OBSOLETING]
-TS_REMOVE_STATES = [TS_ERASE, TS_OBSOLETED]
+TS_REMOVE_STATES = [TS_ERASE, TS_OBSOLETED, TS_UPDATED]
 
 # Transaction Relationships
 TR_UPDATES = 1
@@ -69,6 +74,29 @@ PLUG_OPT_WHERE_MAIN = 0
 PLUG_OPT_WHERE_REPO = 1
 PLUG_OPT_WHERE_ALL = 2
 
+# version of sqlite database schemas
+DBVERSION = '10'
+
 # boolean dict:
 BOOLEAN_STATES = {'1': True, 'yes': True, 'true': True, 'on': True,
                   '0': False, 'no': False, 'false': False, 'off': False}
+
+RPM_TO_SQLITE = { 'packagesize' : 'size_package',
+                 'archivesize' : 'size_archive',
+                 'installedsize' : 'size_installed',
+                 'buildtime' : 'time_build',
+                 'hdrstart' : 'rpm_header_start',
+                 'hdrend' : 'rpm_header_end',
+                 'basepath' : 'location_base',
+                 'relativepath': 'location_href',
+                 'filetime' : 'time_file',
+                 'packager' : 'rpm_packager',
+                 'group' : 'rpm_group',
+                 'buildhost' : 'rpm_buildhost',
+                 'sourcerpm' : 'rpm_sourcerpm',
+                 'vendor' : 'rpm_vendor',
+                 'license' : 'rpm_license'
+                        }
+
+# Cut over for when we should just give up and load everything
+PATTERNS_MAX = 128
