@@ -19,14 +19,14 @@ Depedancy resolution module for yum.
 """
 
 import os.path
-import re
 import types
 import logging
 
 import rpmUtils.transaction
 import rpmUtils.miscutils
 import rpmUtils.arch
-from rpmUtils.arch import archDifference, isMultiLibArch
+from rpmUtils.arch import archDifference, isMultiLibArch, getBestArch
+import misc
 from misc import unique, version_tuple_to_string
 import rpm
 
@@ -35,14 +35,16 @@ from constants import *
 import packages
 import logginglevels
 import Errors
-from i18n import _
 import warnings
 warnings.simplefilter("ignore", Errors.YumFutureDeprecationWarning)
+from operator import itemgetter
+
+from yum import _
 
 try:
-	assert max(2, 4) == 4
+    assert max(2, 4) == 4
 except:
-	# Python-2.4.x doesn't have min/max ... *sigh*
+    # Python-2.4.x doesn't have min/max ... *sigh*
         def min(x, *args): 
             for y in args:
                 if x > y: x = y
@@ -124,6 +126,7 @@ class Depsolve(object):
                             'notriggers': rpm.RPMTRANS_FLAG_NOTRIGGERS,
                             'nodocs': rpm.RPMTRANS_FLAG_NODOCS,
                             'test': rpm.RPMTRANS_FLAG_TEST,
+                            'justdb': rpm.RPMTRANS_FLAG_JUSTDB,
                             'repackage': rpm.RPMTRANS_FLAG_REPACKAGE}
         
         self._ts.setFlags(0) # reset everything.
@@ -149,13 +152,7 @@ class Depsolve(object):
         # /etc/* bin/* or /usr/lib/sendmail then we should fetch the 
         # filelists.xml for all repos to make the searchProvides more complete.
         if name[0] == '/':
-            matched = 0
-            globs = ['.*bin\/.*', '^\/etc\/.*', '^\/usr\/lib\/sendmail$']
-            for glob in globs:
-                globc = re.compile(glob)
-                if globc.match(name):
-                    matched = 1
-            if not matched:
+            if not misc.re_primary_filename(name):
                 self.doSackFilelistPopulate()
             
         pkgs = self.pkgSack.searchProvides(name)
@@ -435,7 +432,7 @@ class Depsolve(object):
         # get rid of things that are already in the rpmdb - b/c it's pointless to use them here
 
         for pkg in provSack.returnPackages():
-            if pkg.pkgtup in self.rpmdb.simplePkgList(): # is it already installed?
+            if self.rpmdb.contains(po=pkg): # is it already installed?
                 self.verbose_logger.log(logginglevels.DEBUG_2, _('%s is in providing packages but it is already installed, removing.'), pkg)
                 provSack.delPackage(pkg)
                 continue
@@ -450,18 +447,18 @@ class Depsolve(object):
                 # from ts
                 tspkgs = self.tsInfo.matchNaevr(name=pkg.name, arch=pkg.arch)
                 for tspkg in tspkgs:
-                    if tspkg.po.EVR > pkg.EVR:
+                    if tspkg.po.verGT(pkg):
                         msg = _('Potential resolving package %s has newer instance in ts.') % pkg
                         self.verbose_logger.log(logginglevels.DEBUG_2, msg)
                         provSack.delPackage(pkg)
                         continue
-                    elif tspkg.po.EVR < pkg.EVR:
+                    elif tspkg.po.verLT(pkg):
                         upgraded.setdefault(pkg.pkgtup, []).append(tspkg.pkgtup)
                 
                 # from rpmdb
                 dbpkgs = self.rpmdb.searchNevra(name=pkg.name, arch=pkg.arch)
                 for dbpkg in dbpkgs:
-                    if dbpkg.EVR > pkg.EVR:
+                    if dbpkg.verGT(pkg):
                         msg = _('Potential resolving package %s has newer instance installed.') % pkg
                         self.verbose_logger.log(logginglevels.DEBUG_2, msg)
                         provSack.delPackage(pkg)
@@ -492,31 +489,23 @@ class Depsolve(object):
         # find the best one 
 
         # try updating the already install pkgs
-        length = len(self.tsInfo)
         for pkg in provSack.returnNewestByName():
-            self.update(requiringPo=requiringPo, name=pkg.name, epoch=pkg.epoch, version=pkg.version, rel=pkg.rel)
-            if len(self.tsInfo) != length:
-                checkdeps = True
-                return checkdeps, missingdep
+            results = self.update(requiringPo=requiringPo, name=pkg.name,
+                                  epoch=pkg.epoch, version=pkg.version,
+                                  rel=pkg.rel)
+            for txmbr in results:
+                if pkg == txmbr.po:
+                    checkdeps = True
+                    return checkdeps, missingdep
 
         # find out which arch of the ones we can choose from is closest
         # to the arch of the requesting pkg
         newest = provSack.returnNewestByNameArch()
         if len(newest) > 1: # there's no way this can be zero
                             
-            best = newest[0]
-            old_best = None
-            loop_run = 0
-            while best != old_best:
-                if loop_run >= len(newest)*2:
-                    msg = _('Failure finding best provider of %s for %s, exceeded maximum loop length' % (needname, requiringPo))
-                    errorlist.append(msg)
-                    self.verbose_logger.debug(msg)
-                    break
-                loop_run += 1
-                old_best = best
-                best = self._compare_providers(newest, best, requiringPo)
-                    
+            pkgresults = self._compare_providers(newest, requiringPo)
+            # take the first one...
+            best = pkgresults[0][0]                   
                 
         elif len(newest) == 1:
             best = newest[0]
@@ -636,9 +625,6 @@ class Depsolve(object):
         if not len(self.tsInfo):
             return (0, [_('Success - empty transaction')])
 
-        # holder object for things from the check
-        if not hasattr(self, '_dcobj'):
-            self._dcobj = DepCheck()
         self.po_with_problems = set()
         self._working_po = None
         self.tsInfo.resetResolved(hard=False)
@@ -765,8 +751,8 @@ class Depsolve(object):
 
 
     def _checkInstall(self, txmbr):
-        reqs = txmbr.po.returnPrco('requires')
-        provs = set(txmbr.po.returnPrco('provides'))
+        txmbr_reqs = txmbr.po.returnPrco('requires')
+        txmbr_provs = set(txmbr.po.returnPrco('provides'))
 
         # if this is an update, we should check what the old
         # requires were to make things faster
@@ -776,10 +762,10 @@ class Depsolve(object):
         oldreqs = set(oldreqs)
 
         ret = []
-        for req in reqs:
+        for req in txmbr_reqs:
             if req[0].startswith('rpmlib('):
                 continue
-            if req in provs:
+            if req in txmbr_provs:
                 continue
             if req in oldreqs and self.rpmdb.getProvides(*req):
                 continue
@@ -787,8 +773,6 @@ class Depsolve(object):
             self.verbose_logger.log(logginglevels.DEBUG_2, _("looking for %s as a requirement of %s"), req, txmbr)
             provs = self.tsInfo.getProvides(*req)
             if not provs:
-                reqtuple = (req[0], version_tuple_to_string(req[2]), flags[req[1]])
-                self._dcobj.addRequires(txmbr.po, [reqtuple])
                 ret.append( (txmbr.po, (req[0], flags[req[1]], version_tuple_to_string(req[2]))) )
                 continue
 
@@ -824,8 +808,6 @@ class Depsolve(object):
             for pkg, hits in self.tsInfo.getRequires(*prov).iteritems():
                 for rn, rf, rv in hits:
                     if not self.tsInfo.getProvides(rn, rf, rv):
-                        reqtuple = (rn, version_tuple_to_string(rv), flags[rf])
-                        self._dcobj.addRequires(pkg, [reqtuple])
                         ret.append( (pkg, (rn, flags[rf], version_tuple_to_string(rv))) )
         return ret
 
@@ -922,14 +904,20 @@ class Depsolve(object):
         return installed
     _isPackageInstalled = isPackageInstalled
 
-    def _compare_providers(self, pkgs, bestpkg, reqpo):
+    def _compare_providers(self, pkgs, reqpo):
+        """take the list of pkgs and score them based on the requesting package
+           return a dictionary of po=score"""
+        self.verbose_logger.log(logginglevels.DEBUG_4,
+              _("Running compare_providers() for %s") %(str(pkgs)))
 
+        
         def _common_prefix_len(x, y, minlen=2):
             num = min(len(x), len(y))
             for off in range(num):
                 if x[off] != y[off]:
                     return max(off, minlen)
             return max(num, minlen)
+
         def _common_sourcerpm(x, y):
             if not hasattr(x, 'sourcerpm'):
                 return False
@@ -937,64 +925,99 @@ class Depsolve(object):
                 return False
             return x.sourcerpm == y.sourcerpm
 
-        for po in pkgs:
-            if po == bestpkg: # if we're comparing the same one, skip it
-                continue
-            # if best is obsoleted by any of the packages, then the obsoleter
-            # is the new best    
-            for obs in po.obsoletes:
-                if bestpkg.inPrcoRange('provides', obs):
-                    return po
-
-            # make sure the best doesn't obsolete this po - if it does we're done
-            # we do this b/c it is possible for two entries to oscillate in this
-            # test - obsolete should trump no matter what
-            # NOTE: mutually obsoleting providers is completely and utterly doom
-            for obs in bestpkg.obsoletes:
-                if po.inPrcoRange('provides', obs):
-                    return bestpkg
-                    
-            if reqpo.arch != 'noarch':
-                best_dist = archDifference(reqpo.arch, bestpkg.arch)
-                if isMultiLibArch(): # only go to the next one if we're multilib - i686 can satisfy i386 deps
-                    if best_dist == 0: # can't really use best's arch anyway...
-                        return po # just try the next one - can't be much worse
-
+        def _compare_arch_distance(x, y, req_compare_arch):
+            # take X and Y package objects
+            # determine which has a closer archdistance to compare_arch
+            # if they are equal to compare_arch, compare which is closer to the 
+            # running arch
+            # return the package which is closer or None for equal, or equally useless
             
-                po_dist = archDifference(reqpo.arch, po.arch)
-                if po_dist > 0 and best_dist > po_dist:
-                    return po
-                    
-                if best_dist == po_dist:
-                    if (not _common_sourcerpm(reqpo, bestpkg) and
-                        _common_sourcerpm(reqpo, po)):
-                        return po
-                    cplp = _common_prefix_len(reqpo.name, po.name)
-                    cplb = _common_prefix_len(reqpo.name, bestpkg.name)
-                    if cplp > cplb:
-                        return po
-                    if cplp == cplb and len(po.name) < len(bestpkg.name):
-                        return po
-                        
-            elif (not _common_sourcerpm(reqpo, bestpkg) and
-                  _common_sourcerpm(reqpo, po)):
-                return po
-            elif (_common_prefix_len(reqpo.name, po.name) >
-                  _common_prefix_len(reqpo.name, bestpkg.name)):
-                return po
-            elif (_common_prefix_len(reqpo.name, po.name) <
-                  _common_prefix_len(reqpo.name, bestpkg.name)):
-                return bestpkg
-            elif len(po.name) < len(bestpkg.name):
-                return po
-            elif len(po.name) == len(bestpkg.name):
-                # compare arch
-                arch = rpmUtils.arch.getBestArchFromList([po.arch, bestpkg.arch])
-                if arch == po.arch:
-                    return po
+            x_dist = archDifference(req_compare_arch, x.arch)
+            if isMultiLibArch(): # only go to the next one if we're multilib - 
+                if x_dist == 0: # can't really use best's arch anyway...
+                    self.verbose_logger.log(logginglevels.DEBUG_4,
+                        _("better arch in po %s") %(y))
+                    return y # just try the next one - can't be much worse
 
-        # Nothing else was better, so this is it
-        return bestpkg
+            y_dist = archDifference(req_compare_arch, y.arch)
+            if y_dist > 0 and x_dist > y_dist:
+                self.verbose_logger.log(logginglevels.DEBUG_4,
+                    _("better arch in po %s") %(y))
+
+                return y
+            if y_dist == x_dist:
+                return None
+            return x
+            
+        pkgresults = {}
+
+        for pkg in pkgs:
+            pkgresults[pkg] = 0
+            
+        # go through each pkg and compare to others
+        # if it is same skip it
+        # if the pkg is obsoleted by any other of the packages
+        # then add  -1024 to its score
+        # don't need to look for mutual obsoletes b/c each package
+        # is evaluated against all the others, so mutually obsoleting
+        # packages will have their scores diminished equally
+        
+        # compare the arch vs each other pkg
+        #   give each time it returns with a better arch a +5
+
+        # look for common source vs the reqpo - give a +10 if it has it
+
+        # look for common_prefix_len - add the length*2 to the score
+        
+        # add the negative of the length of the name to the score
+        
+        for po in pkgs:
+            for nextpo in pkgs:
+                if po == nextpo:
+                    continue
+                obsoleted = False
+                for obs in nextpo.obsoletes:
+                    if po.inPrcoRange('provides', obs):
+                        obsoleted = True
+                                
+                        self.verbose_logger.log(logginglevels.DEBUG_4,
+                            _("%s obsoletes %s") % (po, nextpo))
+
+                    if obsoleted:
+                        pkgresults[po] -= 1024
+                        break
+
+                for thisarch in (reqpo.arch, getBestArch()):
+                    res = _compare_arch_distance(po, nextpo, thisarch)
+                    if not res:
+                        continue
+                    self.verbose_logger.log(logginglevels.DEBUG_4,                   
+                       _('archdist compared %s to %s on %s\n  Winner: %s' % (po, nextpo, thisarch, res)))
+
+                    if res == po:
+                        pkgresults[po] += 5
+
+            if _common_sourcerpm(po, reqpo):
+                self.verbose_logger.log(logginglevels.DEBUG_4,
+                    _('common sourcerpm %s and %s' % (po, reqpo)))
+                pkgresults[po] += 20
+            
+            cpl = _common_prefix_len(po.name, reqpo.name)
+            if cpl > 2:
+                self.verbose_logger.log(logginglevels.DEBUG_4,
+                    _('common prefix of %s between %s and %s' % (cpl, po, reqpo)))
+            
+                pkgresults[po] += cpl*2
+            
+            pkgresults[po] += (len(po.name)*-1)
+
+        bestorder = sorted(pkgresults.items(), key=itemgetter(1), reverse=True)
+        self.verbose_logger.log(logginglevels.DEBUG_4,
+                _('Best Order: %s' % str(bestorder)))
+
+        return bestorder
+                                    
+       
 
 
 class DepCheck(object):
