@@ -90,13 +90,6 @@ def parsePackages(pkgs, usercommands, casematch=0,
        takes an optional casematch option to determine if case should be matched
        exactly. Defaults to not matching."""
 
-    def contains(haystack, needle):
-        """ If anything from needle is in haystack, return True. """
-        for x in needle:
-            if x in haystack:
-                return True
-        return False
-
     pkgdict = buildPkgRefDict(pkgs, bool(casematch))
     exactmatch = []
     matched = []
@@ -110,7 +103,7 @@ def parsePackages(pkgs, usercommands, casematch=0,
         else:
             # anything we couldn't find a match for
             # could mean it's not there, could mean it's a wildcard
-            if contains(command, "*,[]{}?"):
+            if misc.re_glob(command):
                 trylist = pkgdict.keys()
                 # command and pkgdict are already lowered if not casematch
                 # so case sensitive is always fine
@@ -195,17 +188,40 @@ class PackageObject(object):
                                       self.arch)
         return out
 
-    def __cmp__(self, other):
-        """ Compare packages. """
+    def verCMP(self, other):
+        """ Compare package to another one, only rpm-version ordering. """
         if not other:
             return 1
         ret = cmp(self.name, other.name)
         if ret == 0:
             ret = comparePoEVR(self, other)
+        return ret
+
+    def __cmp__(self, other):
+        """ Compare packages, this is just for UI/consistency. """
+        ret = self.verCMP(other)
         if ret == 0:
             ret = cmp(self.arch, other.arch)
+        if ret == 0 and hasattr(self, 'repoid') and hasattr(other, 'repoid'):
+            ret = cmp(self.repoid, other.repoid)
         return ret
 
+    def verEQ(self, other):
+        """ Uses verCMP, tests if the _rpm-versions_ are the same. """
+        return self.verCMP(other) == 0
+    def verLT(self, other):
+        """ Uses verCMP, tests if the other _rpm-version_ is <  ours. """
+        return self.verCMP(other) <  0
+    def verLE(self, other):
+        """ Uses verCMP, tests if the other _rpm-version_ is <= ours. """
+        return self.verCMP(other) <= 0
+    def verGT(self, other):
+        """ Uses verCMP, tests if the other _rpm-version_ is >  ours. """
+        return self.verCMP(other) >  0
+    def verGE(self, other):
+        """ Uses verCMP, tests if the other _rpm-version_ is >= ours. """
+        return self.verCMP(other) >= 0
+
     def __repr__(self):
         return "<%s : %s (%s)>" % (self.__class__.__name__, str(self),hex(id(self))) 
 
@@ -287,10 +303,24 @@ class RpmBase(object):
         # get rid of simple cases - nothing
         if not self.prco.has_key(prcotype):
             return 0
-        # exact match    
-        if prcotuple in self.prco[prcotype]:
-            return 1
+
+        # First try and exact match, then search
+        # Make it faster, if it's "big".
+        if len(self.prco[prcotype]) <= 8:
+            if prcotuple in self.prco[prcotype]:
+                return 1
         else:
+            if not hasattr(self, '_prco_lookup'):
+                self._prco_lookup = {'obsoletes' : None, 'conflicts' : None,
+                                     'requires'  : None, 'provides'  : None}
+
+            if self._prco_lookup[prcotype] is None:
+                self._prco_lookup[prcotype] = set(self.prco[prcotype])
+
+            if prcotuple in self._prco_lookup[prcotype]:
+                return 1
+
+        if True: # Keep indentation for patch smallness...
             # make us look it up and compare
             (reqn, reqf, (reqe, reqv ,reqr)) = prcotuple
             if reqf is not None:
@@ -309,10 +339,7 @@ class RpmBase(object):
         return bool(self.matchingPrcos(prcotype, reqtuple))
 
     def matchingPrcos(self, prcotype, reqtuple):
-        # we only ever get here if we have a versioned prco
-        # nameonly shouldn't ever raise it
         (reqn, reqf, (reqe, reqv, reqr)) = reqtuple
-        # however, just in case
         # find the named entry in pkgobj, do the comparsion
         result = []
         for (n, f, (e, v, r)) in self.returnPrco(prcotype):
@@ -615,6 +642,9 @@ class YumAvailablePackage(PackageObject, RpmBase):
         """return a list of requires in normal rpm format"""
         return self.requires_print
 
+    def returnChecksums(self):
+        return [(self.checksum_type, self.pkgId, 1)]
+        
     def importFromDict(self, pkgdict):
         """handles an mdCache package dictionary item to populate out 
            the package information"""
@@ -709,16 +739,16 @@ class YumHeaderPackage(YumAvailablePackage):
         YumAvailablePackage.__init__(self, repo)
 
         self.hdr = hdr
-        self.name = self.hdr['name']
-        self.arch = self.hdr['arch']
-        self.epoch = self.doepoch()
-        self.version = self.hdr['version']
-        self.release = self.hdr['release']
+        self.name = misc.share_data(self.hdr['name'])
+        self.arch = misc.share_data(self.hdr['arch'])
+        self.epoch = misc.share_data(self.doepoch())
+        self.version = misc.share_data(self.hdr['version'])
+        self.release = misc.share_data(self.hdr['release'])
         self.ver = self.version
         self.rel = self.release
         self.pkgtup = (self.name, self.arch, self.epoch, self.version, self.release)
-        self.summary = self.hdr['summary'].replace('\n', '')
-        self.description = self.hdr['description']
+        self.summary = misc.share_data(self.hdr['summary'].replace('\n', ''))
+        self.description = misc.share_data(self.hdr['description'])
         self.pkgid = self.hdr[rpm.RPMTAG_SHA1HEADER]
         if not self.pkgid:
             self.pkgid = "%s.%s" %(self.hdr['name'], self.hdr['buildtime'])
@@ -741,25 +771,34 @@ class YumHeaderPackage(YumAvailablePackage):
             self.__prcoPopulated = True
         return YumAvailablePackage.returnPrco(self, prcotype, printable)
 
+    def _get_hdr(self):
+        return self.hdr
+
     def _populatePrco(self):
         "Populate the package object with the needed PRCO interface."
 
-        tag2prco = { "OBSOLETE": "obsoletes",
-                     "CONFLICT": "conflicts",
-                     "REQUIRE": "requires",
-                     "PROVIDE": "provides" }
+        tag2prco = { "OBSOLETE": misc.share_data("obsoletes"),
+                     "CONFLICT": misc.share_data("conflicts"),
+                     "REQUIRE":  misc.share_data("requires"),
+                     "PROVIDE":  misc.share_data("provides") }
+        hdr = self._get_hdr()
         for tag in tag2prco:
-            name = self.hdr[getattr(rpm, 'RPMTAG_%sNAME' % tag)]
+            name = hdr[getattr(rpm, 'RPMTAG_%sNAME' % tag)]
+            name = map(misc.share_data, name)
+            if name is None:
+                continue
 
-            lst = self.hdr[getattr(rpm, 'RPMTAG_%sFLAGS' % tag)]
+            lst = hdr[getattr(rpm, 'RPMTAG_%sFLAGS' % tag)]
             flag = map(rpmUtils.miscutils.flagToString, lst)
+            flag = map(misc.share_data, flag)
 
-            lst = self.hdr[getattr(rpm, 'RPMTAG_%sVERSION' % tag)]
+            lst = hdr[getattr(rpm, 'RPMTAG_%sVERSION' % tag)]
             vers = map(rpmUtils.miscutils.stringToVersion, lst)
+            vers = map(lambda x: (misc.share_data(x[0]), misc.share_data(x[1]),
+                                  misc.share_data(x[2])), vers)
 
             prcotype = tag2prco[tag]
-            if name is not None:
-                self.prco[prcotype] = zip(name, flag, vers)
+            self.prco[prcotype] = map(misc.share_data, zip(name,flag,vers))
     
     def tagByName(self, tag):
         warnings.warn("tagByName() will go away in a furture version of Yum.\n",
@@ -770,6 +809,8 @@ class YumHeaderPackage(YumAvailablePackage):
             raise Errors.MiscError, "Unknown header tag %s" % tag
 
     def __getattr__(self, thing):
+        #FIXME - if an error - return AttributeError, not KeyError 
+        # ONLY FIX THIS AFTER THE API BREAK
         return self.hdr[thing]
 
     def doepoch(self):
@@ -836,6 +877,9 @@ class YumHeaderPackage(YumAvailablePackage):
                        self.hdr['changelogtext'])
         return []
 
+    def returnChecksums(self):
+        raise NotImplementedError()
+        
 class _CountedReadFile:
     """ Has just a read() method, and keeps a count so we can find out how much
         has been read. Implemented so we can get the real size of the file from
@@ -1164,6 +1208,14 @@ class YumLocalPackage(YumHeaderPackage):
         self._stat = os.stat(self.localpath)
         self.filetime = str(self._stat[-1])
         self.packagesize = str(self._stat[6])
+        self.arch = self.isSrpm()
+        self.pkgtup = (self.name, self.arch, self.epoch, self.ver, self.rel)
+        
+    def isSrpm(self):
+        if self.tagByName('sourcepackage') == 1 or not self.tagByName('sourcerpm'):
+            return 'src'
+        else:
+            return self.tagByName('arch')
         
     def localPkg(self):
         return self.localpath
