@@ -31,8 +31,11 @@ import Errors
 from parser import ConfigPreProcessor
 
 from textwrap import fill
-from i18n import _
+import fnmatch
 
+from weakref import proxy as weakref
+
+from yum import _
 
 # TODO: expose rpm package sack objects to plugins (once finished)
 # TODO: allow plugins to use the existing config stuff to define options for
@@ -130,7 +133,7 @@ class YumPlugins:
 
         self.searchpath = searchpath
         self.pluginconfpath = pluginconfpath
-        self.base = base
+        self.base = weakref(base)
         self.optparser = optparser
         self.cmdline = (None, None)
         self.verbose_logger = logging.getLogger("yum.verbose.YumPlugins")
@@ -182,21 +185,29 @@ class YumPlugins:
             self._pluginfuncs[slot] = []
 
         # Import plugins 
+        self._used_disable_plugin = set()
         for dir in self.searchpath:
             if not os.path.isdir(dir):
                 continue
             for modulefile in glob.glob('%s/*.py' % dir):
                 self._loadplugin(modulefile, types)
-            plugins = sorted(self._plugins)
 
+        if self._plugins:
             # Mostly copied from YumOutput._outKeyValFill()
             key = _("Loaded plugins: ")
-            val = ", ".join(plugins)
+            val = ", ".join(sorted(self._plugins))
             nxt = ' ' * (len(key) - 2) + ': '
             self.verbose_logger.log(logginglevels.INFO_2,
                                     fill(val, width=80, initial_indent=key,
                                          subsequent_indent=nxt))
 
+        if self.disabledPlugins:
+            for wc in self.disabledPlugins:
+                if wc not in self._used_disable_plugin:
+                    self.verbose_logger.log(logginglevels.INFO_2,
+                                            _("No plugin match for: %s") % wc)
+        del self._used_disable_plugin
+
     def _loadplugin(self, modulefile, types):
         '''Attempt to import a plugin module and register the hook methods it
         uses.
@@ -247,8 +258,10 @@ class YumPlugins:
                 return
         # Check if this plugin has been temporary disabled
         if self.disabledPlugins:
-            if modname in self.disabledPlugins:
-                return
+            for wc in self.disabledPlugins:
+                if fnmatch.fnmatch(modname, wc):
+                    self._used_disable_plugin.add(wc)
+                    return
 
         self.verbose_logger.log(logginglevels.DEBUG_3, _('Loading "%s" plugin'),
                                 modname)
@@ -287,7 +300,7 @@ class YumPlugins:
         confpp_obj = ConfigPreProcessor(conffilename)
         try:
             parser.readfp(confpp_obj)
-        except ParsingError, e:
+        except ConfigParser.ParsingError, e:
             raise Errors.ConfigError("Couldn't parse %s: %s" % (conffilename,
                 str(e)))
         return parser
