@@ -21,7 +21,6 @@
 
 import os
 import os.path
-import re
 import fnmatch
 
 import yumRepo
@@ -32,25 +31,25 @@ import misc
 from sqlutils import executeSQL
 import rpmUtils.miscutils
 import sqlutils
+import constants
+import operator
+import time
 
 def catchSqliteException(func):
     """This decorator converts sqlite exceptions into RepoError"""
     def newFunc(*args, **kwargs):
         try:
             return func(*args, **kwargs)
-        except sqlutils.sqlite.Error:
-            raise Errors.RepoError
+        except sqlutils.sqlite.Error, e:
+            raise Errors.RepoError, str(e.message)
 
     newFunc.__name__ = func.__name__
     newFunc.__doc__ = func.__doc__
     newFunc.__dict__.update(func.__dict__)
     return newFunc
 
-_store = {}
 def _share_data(value):
-    """ Take a value and use the same value from the store,
-        if the value isn't in the store this one becomes the shared version. """
-    return _store.setdefault(value, value)
+    return misc.share_data(value)
 
 class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
     def __init__(self, repo, db_obj):
@@ -64,6 +63,10 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         self.state = None
         self._loadedfiles = False
         self._read_db_obj(db_obj)
+        # for stupid metadata created without epochs listed in the version tag
+        # die die
+        if self.epoch is None:
+            self.epoch = '0'
         self.id = self.pkgId
         self.ver = self.version 
         self.rel = self.release 
@@ -83,13 +86,13 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
             except (IndexError, KeyError):
                 return None
 
-        for item in ['name', 'arch', 'epoch', 'version', 'release']:
+        for item in ['name', 'arch', 'epoch', 'version', 'release', 'pkgKey']:
             try:
                 setattr(self, item, _share_data(db_obj[item]))
             except (IndexError, KeyError):
                 pass
 
-        for item in ['pkgId', 'pkgKey']:
+        for item in ['pkgId']:
             try:
                 setattr(self, item, db_obj[item])
             except (IndexError, KeyError):
@@ -133,12 +136,22 @@ class YumAvailablePackageSqlite(YumAvailablePackage, PackageObject, RpmBase):
         dbname = varname
         if db2simplemap.has_key(varname):
             dbname = db2simplemap[varname]
-        r = self._sql_MD('primary',
+        try:
+            r = self._sql_MD('primary',
                          "SELECT %s FROM packages WHERE pkgId = ?" % dbname,
                          (self.pkgId,)).fetchone()
+        except Errors.RepoError, e:
+            if str(e).startswith('no such column'):
+                #FIXME - after API break make this an AttributeError Raise
+                raise KeyError, str(e)
+            raise                         
         value = r[0]
+        if varname == 'epoch' and value is None:
+            value = '0'
         if varname in {'vendor' : 1, 'packager' : 1, 'buildhost' : 1,
-                       'license' : 1, 'group' : 1}:
+                       'license' : 1, 'group' : 1,
+                       'summary' : 1, 'description' : 1, 'sourcerpm' : 1,
+                       'url' : 1}:
             value  = _share_data(value)
         setattr(self, varname, value)
             
@@ -285,8 +298,24 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             pkg_num += self._sql_MD_pkg_num('primary', repo)
         return pkg_num - exclude_num
 
+    def dropCachedData(self):
+        if hasattr(self, '_memoize_requires'):
+            del self._memoize_requires
+        if hasattr(self, '_memoize_provides'):
+            del self._memoize_provides
+        if hasattr(self, 'pkgobjlist'):
+            del self.pkgobjlist
+        self._key2pkg = {}
+        self._search_cache = {
+            'provides' : { },
+            'requires' : { },
+            }
+        misc.unshare_data()
+
     @catchSqliteException
     def close(self):
+        self.dropCachedData()
+
         for dataobj in self.primarydb.values() + \
                        self.filelistsdb.values() + \
                        self.otherdb.values():
@@ -297,12 +326,6 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         self.excludes = {}
         self._excludes = set()
         self._all_excludes = {}
-        self._search_cache = {
-            'provides' : { },
-            'requires' : { },
-            }
-        if hasattr(self, 'pkgobjlist'):
-            del self.pkgobjlist
 
         yumRepo.YumPackageSack.close(self)
 
@@ -326,6 +349,10 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     def _delAllPackages(self, repo):
         """ Exclude all packages from the repo. """
         self._all_excludes[repo] = True
+        if repo in self.excludes:
+            del self.excludes[repo]
+        if repo in self._key2pkg:
+            del self._key2pkg[repo]
 
     def _excluded(self, repo, pkgId):
         if repo in self._all_excludes:
@@ -346,6 +373,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return self._pkgKeyExcluded(po.repo, po.pkgKey)
 
     def _packageByKey(self, repo, pkgKey):
+        """ Lookup a pkg by it's pkgKey, if we don't have it load it """
         if not self._key2pkg.has_key(repo):
             self._key2pkg[repo] = {}
         if not self._key2pkg[repo].has_key(pkgKey):
@@ -356,6 +384,13 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             self._key2pkg[repo][pkgKey] = po
         return self._key2pkg[repo][pkgKey]
         
+    def _packageByKeyData(self, repo, pkgKey, data):
+        """ Like _packageByKey() but we already have the data for .pc() """
+        if data['pkgKey'] not in self._key2pkg.get(repo, {}):
+            po = self.pc(repo, data)
+            self._key2pkg.setdefault(repo, {})[pkgKey] = po
+        return self._key2pkg[repo][data['pkgKey']]
+
     def addDict(self, repo, datatype, dataobj, callback=None):
         if self.added.has_key(repo):
             if datatype in self.added[repo]:
@@ -366,6 +401,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if not self.excludes.has_key(repo): 
             self.excludes[repo] = {}
 
+        if dataobj is None:
+            raise Errors.RepoError, "Tried to add None %s to %s" % (datatype, repo)
+
         if datatype == 'metadata':
             self.primarydb[repo] = dataobj
         elif datatype == 'filelists':
@@ -374,7 +412,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             self.otherdb[repo] = dataobj
         else:
             # We can not handle this yet...
-            raise "Sorry sqlite does not support %s" % (datatype)
+            raise Errors.RepoError, "Sorry sqlite does not support %s in %s" % (datatype, repo)
     
         self.added[repo].append(datatype)
 
@@ -386,15 +424,29 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         # this function is just silly and it reduces down to just this
         return self.searchPrco(name, 'provides')
 
-    def _sql_pkgKey2po(self, repo, cur, pkgs=None):
+    def _sql_pkgKey2po(self, repo, cur, pkgs=None, have_data=False):
         """ Takes a cursor and maps the pkgKey rows into a list of packages. """
         if pkgs is None: pkgs = []
         for ob in cur:
             if self._pkgKeyExcluded(repo, ob['pkgKey']):
                 continue
-            pkgs.append(self._packageByKey(repo, ob['pkgKey']))
+            if have_data:
+                pkgs.append(self._packageByKeyData(repo, ob['pkgKey'], ob))
+            else:
+                pkgs.append(self._packageByKey(repo, ob['pkgKey']))
         return pkgs
 
+    @staticmethod
+    def _sql_esc(pattern):
+        """ Apply SQLite escaping, if needed. Returns pattern and esc. """
+        esc = ''
+        if "_" in pattern or "%" in pattern:
+            esc = " ESCAPE '!'"
+            pattern = pattern.replace("!", "!!")
+            pattern = pattern.replace("%", "!%")
+            pattern = pattern.replace("_", "!_")
+        return (pattern, esc)
+
     @catchSqliteException
     def searchFiles(self, name, strict=False):
         """search primary if file will be in there, if not, search filelists, use globs, if possible"""
@@ -405,7 +457,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         
         glob = True
         querytype = 'glob'
-        if strict or not re.match('.*[\*\?\[\]].*', name):
+        if strict or not misc.re_glob(name):
             glob = False
             querytype = '='
 
@@ -418,6 +470,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             # grab repo object from primarydb and force filelists population in this sack using repo
             # sack.populate(repo, mdtype, callback, cacheonly)
             for (repo,cache) in self.primarydb.items():
+                if repo in self._all_excludes:
+                    continue
+
                 self.populate(repo, mdtype='filelists')
 
         # Check to make sure the DB data matches, this should always pass but
@@ -429,6 +484,9 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 raise Errors.RepoError
 
         for (rep,cache) in self.filelistsdb.items():
+            if rep in self._all_excludes:
+                continue
+
             cur = cache.cursor()
 
             if glob:
@@ -473,14 +531,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if len(fields) < 1:
             return result
         
-        basestring="select DISTINCT pkgKey from packages where %s like '%%%s%%' " % (fields[0], searchstring)
+        searchstring = searchstring.replace("'", "''")
+        (searchstring, esc) = self._sql_esc(searchstring)
+        sql = "select DISTINCT pkgKey from packages where %s like '%%%s%%'%s " % (fields[0], searchstring, esc)
         
         for f in fields[1:]:
-            basestring = "%s or %s like '%%%s%%' " % (basestring, f, searchstring)
+            sql = "%s or %s like '%%%s%%'%s " % (sql, f, searchstring, esc)
         
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            executeSQL(cur, basestring)
+            executeSQL(cur, sql)
             self._sql_pkgKey2po(rep, cur, result)
         return result    
 
@@ -493,6 +553,17 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if len(fields) < 1:
             return result
         
+        # NOTE: I can't see any reason not to use this all the time, speed
+        # comparison shows them as baiscally equal.
+        if len(searchstrings) > constants.PATTERNS_MAX:
+            tot = {}
+            for searchstring in searchstrings:
+                matches = self.searchPrimaryFields(fields, searchstring)
+                for po in matches:
+                    tot[po] = tot.get(po, 0) + 1
+            for po in sorted(tot, key=operator.itemgetter, reverse=True):
+                result.append((po, tot[po]))
+            return result
        
         unionstring = "select pkgKey, SUM(cumul) AS total from ( "
         endunionstring = ")GROUP BY pkgKey ORDER BY total DESC"
@@ -502,23 +573,16 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         #GROUP BY pkgkey ORDER BY total DESC;
         selects = []
         
-        # select pkgKey, 1 AS cumul from packages where description 
-        # like '%devel%' or description like '%python%' or description like '%ssh%'
-#        for f in fields:
-#            basestring = "select pkgKey, 1 AS cumul from packages where %s like '%%%s%%' " % (f,searchstrings[0]) 
-#            for s in searchstrings[1:]:
-#                basestring = "%s or %s like '%%%s%%' " % (basestring, f, s)
-#            selects.append(basestring)
-            
         for s in searchstrings:         
-            basestring="select pkgKey,1 AS cumul from packages where %s like '%%%s%%' " % (fields[0], s)
+            s = s.replace("'", "''")
+            (s, esc) = self._sql_esc(s)
+            sql="select pkgKey,1 AS cumul from packages where %s like '%%%s%%'%s " % (fields[0], s, esc)
             for f in fields[1:]:
-                basestring = "%s or %s like '%%%s%%' " % (basestring, f, s)
-            selects.append(basestring)
+                sql = "%s or %s like '%%%s%%'%s " % (sql, f, s, esc)
+            selects.append(sql)
         
         totalstring = unionstring + " UNION ALL ".join(selects) + endunionstring
-#        print totalstring
-        
+
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
             executeSQL(cur, totalstring)
@@ -589,6 +653,28 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         
         return pkgs
         
+    @catchSqliteException
+    def _search_get_memoize(self, prcotype):
+        if not hasattr(self, '_memoize_' + prcotype):
+            memoize = {}
+
+            for (rep,cache) in self.primarydb.items():
+                if rep in self._all_excludes:
+                    continue
+
+                cur = cache.cursor()
+                executeSQL(cur, "select * from %s" % prcotype)
+                for x in cur:
+                    val = (_share_data(x['name']), _share_data(x['flags']),
+                           (_share_data(x['epoch']), _share_data(x['version']),
+                            _share_data(x['release'])))
+                    val = _share_data(val)
+                    key = (rep, val[0])
+                    pkgkey = _share_data(x['pkgKey'])
+                    val = (pkgkey, val)
+                    memoize.setdefault(key, []).append(val)
+            setattr(self, '_memoize_' + prcotype, memoize)
+        return getattr(self, '_memoize_' + prcotype)
 
     @catchSqliteException
     def _search(self, prcotype, name, flags, version):
@@ -600,12 +686,43 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         elif type(version) in (tuple, list): # would this ever be a list?
             req = (name, flags, version)
 
+        prcotype = _share_data(prcotype)
+        req      = _share_data(req)
         if self._search_cache[prcotype].has_key(req):
             return self._search_cache[prcotype][req]
 
         result = { }
 
-        for (rep,cache) in self.primarydb.items():
+        #  Requires is the biggest hit, pre-loading provides actually hurts
+        #  NOTE: Disabling atm. ... small install/updates get a significant hit.
+        # And even large updates take a hit with the memoize path, maybe we
+        # fixed something with later change? ... maybe I was on crack?
+        #  Speed seems to depend on _search_cache.
+        if True: # prcotype != 'requires':
+            primarydb_items = self.primarydb.items()
+            preload = False
+        else:
+            primarydb_items = []
+            preload = True
+            memoize = self._search_get_memoize(prcotype)
+            for (rep,cache) in self.primarydb.items():
+                if rep in self._all_excludes:
+                    continue
+
+                tmp = {}
+                for x in memoize.get((rep, name), []):
+                    pkgkey, val = x
+                    if rpmUtils.miscutils.rangeCompare(req, val):
+                        tmp.setdefault(pkgkey, []).append(val)
+                for pkgKey, hits in tmp.iteritems():
+                    if self._pkgKeyExcluded(rep, pkgKey):
+                        continue
+                    result[self._packageByKey(rep, pkgKey)] = hits
+
+        for (rep,cache) in primarydb_items:
+            if rep in self._all_excludes:
+                continue
+
             cur = cache.cursor()
             executeSQL(cur, "select * from %s where name=?" % prcotype,
                        (name,))
@@ -623,25 +740,24 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                 result[self._packageByKey(rep, pkgKey)] = hits
 
         if prcotype != 'provides' or name[0] != '/':
-            self._search_cache[prcotype][req] = result
+            if not preload:
+                self._search_cache[prcotype][req] = result
             return result
 
-        matched = 0
-        globs = ['.*bin\/.*', '^\/etc\/.*', '^\/usr\/lib\/sendmail$']
-        for thisglob in globs:
-            globc = re.compile(thisglob)
-            if globc.match(name):
-                matched = 1
-
-        if not matched: # if its not in the primary.xml files
+        if not misc.re_primary_filename(name):
+            # if its not in the primary.xml files
             # search the files.xml file info
             for pkg in self.searchFiles(name, strict=True):
                 result[pkg] = [(name, None, None)]
-            self._search_cache[prcotype][req] = result
+            if not preload:
+                self._search_cache[prcotype][req] = result
             return result
 
         # If it is a filename, search the primary.xml file info
         for (rep,cache) in self.primarydb.items():
+            if rep in self._all_excludes:
+                continue
+
             cur = cache.cursor()
             executeSQL(cur, "select DISTINCT pkgKey from files where name = ?", (name,))
             for ob in cur:
@@ -657,13 +773,33 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
     def getRequires(self, name, flags=None, version=(None, None, None)):
         return self._search("requires", name, flags, version)
 
-    
+    @catchSqliteException
+    def searchNames(self, names):
+        """return a list of packages matching any of the given names. This is 
+           only a match on package name, nothing else"""
+        
+        pat_sqls = []
+        qsql = """select pkgId,pkgKey,name,epoch,version,release,arch
+                      from packages where """
+        for name in names:
+            pat_sqls.append("name = ?")
+        qsql = qsql + " OR ".join(pat_sqls)
+
+        returnList = []
+        for (repo, cache) in self.primarydb.items():
+            cur = cache.cursor()
+            executeSQL(cur, qsql, list(names))
+
+            self._sql_pkgKey2po(repo, cur, returnList, have_data=True)
+
+        return returnList
+ 
     @catchSqliteException
     def searchPrco(self, name, prcotype):
         """return list of packages having prcotype name (any evr and flag)"""
         glob = True
         querytype = 'glob'
-        if not re.match('.*[\*\?\[\]].*', name):
+        if not misc.re_glob(name):
             glob = False
             querytype = '='
 
@@ -674,24 +810,19 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             self._sql_pkgKey2po(rep, cur, results)
         
         # If it's not a provides or a filename, we are done
-        if prcotype != "provides" or name[0] != '/':
-            if not glob:
-                return results
+        if prcotype != "provides":
+            return results
+        if not glob and name[0] != '/':
+            return results
 
         # If it is a filename, search the primary.xml file info
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
             executeSQL(cur, "select DISTINCT pkgKey from files where name %s ?" % querytype, (name,))
             self._sql_pkgKey2po(rep, cur, results)
-        
-        matched = 0
-        globs = ['.*bin\/.*', '^\/etc\/.*', '^\/usr\/lib\/sendmail$']
-        for thisglob in globs:
-            globc = re.compile(thisglob)
-            if globc.match(name):
-                matched = 1
 
-        if matched and not glob: # if its in the primary.xml files then skip the other check
+        # if its in the primary.xml files then skip the other check
+        if misc.re_primary_filename(name) and not glob:
             return misc.unique(results)
 
         # If it is a filename, search the files.xml file info
@@ -788,20 +919,21 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return y
 
     @catchSqliteException
-    def returnNewestByNameArch(self, naTup=None, patterns=None):
+    def returnNewestByNameArch(self, naTup=None, patterns=None, ignore_case=False):
 
         # If naTup is set do it from the database otherwise use our parent's
         # returnNewestByNameArch
         if (not naTup):
             return yumRepo.YumPackageSack.returnNewestByNameArch(self, naTup,
-                                                                 patterns)
+                                                                 patterns,
+                                                                 ignore_case)
 
         # First find all packages that fulfill naTup
         allpkg = []
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            executeSQL(cur, "select pkgKey from packages where name=? and arch=?",naTup)
-            self._sql_pkgKey2po(rep, cur, allpkg)
+            executeSQL(cur, "select pkgId,pkgKey,name,epoch,version,release,arch from packages where name=? and arch=?", naTup)
+            self._sql_pkgKey2po(rep, cur, allpkg, have_data=True)
         
         # if we've got zilch then raise
         if not allpkg:
@@ -809,18 +941,20 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return misc.newestInList(allpkg)
 
     @catchSqliteException
-    def returnNewestByName(self, name=None):
+    def returnNewestByName(self, name=None, patterns=None, ignore_case=False):
         # If name is set do it from the database otherwise use our parent's
         # returnNewestByName
         if (not name):
-            return yumRepo.YumPackageSack.returnNewestByName(self, name)
+            return yumRepo.YumPackageSack.returnNewestByName(self, name,
+                                                             patterns,
+                                                             ignore_case)
 
         # First find all packages that fulfill name
         allpkg = []
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
-            executeSQL(cur, "select pkgKey from packages where name=?", (name,))
-            self._sql_pkgKey2po(rep, cur, allpkg)
+            executeSQL(cur, "select pkgId,pkgKey,name,epoch,version,release,arch from packages where name=?", (name,))
+            self._sql_pkgKey2po(rep, cur, allpkg, have_data=True)
         
         # if we've got zilch then raise
         if not allpkg:
@@ -835,7 +969,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         unmatched = list(pkgspecs)
 
         for p in pkgspecs:
-            if re.match('.*[\*\?\[\]].*', p):
+            if misc.re_glob(p):
                 query = PARSE_QUERY % ({ "op": "glob", "q": p })
                 matchres = matched
             else:
@@ -856,11 +990,11 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         return exactmatch, matched, unmatched
 
     @catchSqliteException
-    def _buildPkgObjList(self, repoid=None, patterns=None):
+    def _buildPkgObjList(self, repoid=None, patterns=None, ignore_case=False):
         """Builds a list of packages, only containing nevra information. No
            excludes are done at this stage. """
 
-        if patterns is None:
+        if patterns is None or len(patterns) > constants.PATTERNS_MAX:
             patterns = []
         
         returnList = []        
@@ -877,23 +1011,25 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
                     for field in ['name', 'sql_nameArch', 'sql_nameVerRelArch',
                                   'sql_nameVer', 'sql_nameVerRel',
                                   'sql_envra', 'sql_nevra']:
-                        pat_sqls.append("%s GLOB ?" % field)
+                        if ignore_case:
+                            (pattern, esc) = self._sql_esc(pattern)
+                            pattern = pattern.replace("*", "%")
+                            pattern = pattern.replace("?", "_")
+                            pat_sqls.append("%s LIKE ?%s" % (field, esc))
+                        else:
+                            pat_sqls.append("%s GLOB ?" % field)
                         pat_data.append(pattern)
                 if pat_sqls:
                     qsql = _FULL_PARSE_QUERY_BEG + " OR ".join(pat_sqls)
                 executeSQL(cur, qsql, pat_data)
                 for x in cur:
-                    if self._key2pkg.get(repo, {}).has_key(x['pkgKey']):
-                        po = self._key2pkg[repo][x['pkgKey']]
-                    else:
-                        po = self.pc(repo,x)
-                        self._key2pkg.setdefault(repo, {})[po.pkgKey] = po
+                    po = self._packageByKeyData(repo, x['pkgKey'], x)
                     returnList.append(po)
         if not patterns:
             self.pkgobjlist = returnList
         return returnList
                 
-    def returnPackages(self, repoid=None, patterns=None):
+    def returnPackages(self, repoid=None, patterns=None, ignore_case=False):
         """Returns a list of packages, only containing nevra information. The
            packages are processed for excludes. Note that patterns is just
            a hint, we are free it ignore it. """
@@ -910,7 +1046,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         if hasattr(self, 'pkgobjlist'):
             pkgobjlist = self.pkgobjlist
         else:
-            pkgobjlist = self._buildPkgObjList(repoid, patterns)
+            pkgobjlist = self._buildPkgObjList(repoid, patterns, ignore_case)
 
         returnList = []
         for po in pkgobjlist:
@@ -934,7 +1070,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
             return returnList
         
         # make up our execute string
-        q = "select pkgKey from packages WHERE"
+        q = "select pkgId,pkgKey,name,epoch,version,release,arch from packages WHERE"
         for (col, var) in [('name', name), ('epoch', epoch), ('version', ver),
                            ('arch', arch), ('release', rel)]:
             if var:
@@ -947,7 +1083,7 @@ class YumSqlitePackageSack(yumRepo.YumPackageSack):
         for (rep,cache) in self.primarydb.items():
             cur = cache.cursor()
             executeSQL(cur, q)
-            self._sql_pkgKey2po(rep, cur, returnList)
+            self._sql_pkgKey2po(rep, cur, returnList, have_data=True)
         return returnList
     
     @catchSqliteException
@@ -1020,15 +1156,17 @@ where name %(op)s '%(q)s'
    or name || '-' || epoch || ':' || version || '-' || release || '.' || arch %(op)s '%(q)s'
 """
 
-# This is roughly the same as above, and used by _buildPkgObjList()
+# This is roughly the same as above, and used by _buildPkgObjList().
+#  Use " to quote because we using ? ... and sqlutils.QmarkToPyformat gets
+# confused.
 _FULL_PARSE_QUERY_BEG = """
 SELECT pkgId,pkgKey,name,epoch,version,release,arch,
-  name || '.' || arch AS sql_nameArch,
-  name || '-' || version || '-' || release || '.' || arch AS sql_nameVerRelArch,
-  name || '-' || version AS sql_nameVer,
-  name || '-' || version || '-' || release AS sql_nameVerRel,
-  epoch || ':' || name || '-' || version || '-' || release || '.' || arch AS sql_envra,
-  name || '-' || epoch || ':' || version || '-' || release || '.' || arch AS sql_nevra
+  name || "." || arch AS sql_nameArch,
+  name || "-" || version || "-" || release || "." || arch AS sql_nameVerRelArch,
+  name || "-" || version AS sql_nameVer,
+  name || "-" || version || "-" || release AS sql_nameVerRel,
+  epoch || ":" || name || "-" || version || "-" || release || "." || arch AS sql_envra,
+  name || "-" || epoch || ":" || version || "-" || release || "." || arch AS sql_nevra
   FROM packages
   WHERE
 """
