@@ -22,9 +22,10 @@ Classes for subcommands of the yum command line interface.
 import os
 import cli
 from yum import logginglevels
+from yum import _
 import yum.Errors
-from yum.i18n import _
-
+import operator
+import locale
 
 def checkRootUID(base):
     """
@@ -78,7 +79,7 @@ def checkGroupArg(base, basecmd, extcmds):
 
 def checkCleanArg(base, basecmd, extcmds):
     VALID_ARGS = ('headers', 'packages', 'metadata', 'dbcache', 'plugins',
-            'all')
+                  'expire-cache', 'all')
 
     if len(extcmds) == 0:
         base.logger.critical(_('Error: clean requires an option: %s') % (
@@ -219,7 +220,9 @@ class InfoCommand(YumCommand):
             # if we've looked up obsolete lists and it's a list request
                 rop = [0, '']
                 print _('Obsoleting Packages')
-                for obtup in ypl.obsoletesTuples:
+                # The tuple is (newPkg, oldPkg) ... so sort by new
+                for obtup in sorted(ypl.obsoletesTuples,
+                                    key=operator.itemgetter(0)):
                     base.updatesObsoletesList(obtup, 'obsoletes')
             else:
                 rop = base.listPkgs(ypl.obsoletes, _('Obsoleting Packages'), basecmd)
@@ -233,7 +236,7 @@ class InfoCommand(YumCommand):
             return 0, []
 
     def needTs(self, base, basecmd, extcmds):
-        if len(extcmds) and extcmds == ['installed']:
+        if len(extcmds) and extcmds[0] == 'installed':
             return False
         
         return True
@@ -420,7 +423,7 @@ class CleanCommand(YumCommand):
         return ['clean']
 
     def getUsage(self):
-        return "[headers|packages|metadata|dbcache|plugins|all]"
+        return "[headers|packages|metadata|dbcache|plugins|expire-cache|all]"
 
     def getSummary(self):
         return _("Remove cached data")
@@ -639,21 +642,60 @@ class RepoListCommand(YumCommand):
         else:
             arg = 'enabled'
 
-        format_string = "%-20.20s %-40.40s  %s"
-        if base.repos.repos.values():
-            base.verbose_logger.log(logginglevels.INFO_2, format_string,
-                _('repo id'), _('repo name'), _('status'))
+        # Setup so len(repo.sack) is correct
+        base.repos.populateSack()
+
+        format_string = "%-20.20s %-40.40s %-8s%s"
         repos = base.repos.repos.values()
         repos.sort()
+        enabled_repos = base.repos.listEnabled()
+        done = False
+        verbose = base.verbose_logger.isEnabledFor(logginglevels.DEBUG_3)
+        if arg == 'all':
+            ehibeg = base.term.FG_COLOR['green'] + base.term.MODE['bold']
+            dhibeg = base.term.FG_COLOR['red']
+            hiend  = base.term.MODE['normal']
+        else:
+            ehibeg = ''
+            dhibeg = ''
+            hiend  = ''
+        tot_num = 0
         for repo in repos:
-            if repo in base.repos.listEnabled() and arg in ('all', 'enabled'):
-                base.verbose_logger.log(logginglevels.INFO_2, format_string,
-                    repo, repo.name, _('enabled'))
-            elif arg in ('all', 'disabled'):
-                base.verbose_logger.log(logginglevels.INFO_2, format_string,
-                    repo, repo.name, _('disabled'))
+            if repo in enabled_repos:
+                enabled = True
+                ui_enabled = ehibeg + _('enabled') + hiend
+                num        = len(repo.sack)
+                tot_num   += num
+                ui_num     = locale.format("%d", num, True)
+                ui_fmt_num = ": %7s"
+            else:
+                enabled = False
+                ui_enabled = dhibeg + _('disabled') + hiend
+                ui_num     = ""
+                ui_fmt_num = "%s"
+                
+            if (arg == 'all' or
+                (arg == 'enabled' and enabled) or
+                (arg == 'disabled' and not enabled)):
+                if not done and not verbose:
+                    base.verbose_logger.log(logginglevels.INFO_2,
+                                            format_string, _('repo id'),
+                                            _('repo name'), _('status'))
+                done = True
+                if verbose:
+                    line1 = base.fmtKeyValFill(_("Repo-id     : "), repo)
+                    line2 = base.fmtKeyValFill(_("Repo-name   : "), repo.name)
+                    line3 = base.fmtKeyValFill(_("Repo-enabled: "), ui_enabled)
+                    line4 = base.fmtKeyValFill(_("Repo-size   : "), ui_num)
+                    base.verbose_logger.log(logginglevels.DEBUG_3,
+                                            "%s\n%s\n%s\n%s\n",
+                                            line1, line2, line3, line4)
+                else:
+                    base.verbose_logger.log(logginglevels.INFO_2, format_string,
+                                            repo, repo.name, ui_enabled,
+                                            ui_fmt_num % ui_num)
 
-        return 0, []
+        return 0, ['repolist: ' + locale.format("%d", tot_num, True)]
 
     def needTs(self, base, basecmd, extcmds):
         return False
